#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 04-Jun-2003 
#################################################################################
#
# Home - Home page component for Ethereal Realms 
# Copyright (C) 2000-2004  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 29-26503 TWP RD 511
#                 Spruce Grove, Alberta
#                 T7Y 1G4
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp;							# CGI Error logs
use File::Copy;							# File copy handle
use strict;							# Strict variable enforcement

use eRealms::Database;						# Database handler
use eRealms::Home;						# Homepage/file component
use eRealms::Image;						# Image handling library
use eRealms::Login;						# Login functionality
use eRealms::Param;						# Parameter control
use eRealms::Table;						# Table creation
use eRealms::Template;						# Template handling


#################################################################################
# Data Members
#################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $hp;								# Homepage component
my $login;							# Login handle
my $param;							# Parameter handle
my $table;							# Table handler
my $tmpl;							# Template handle

my $expire;							# Expired homepages
my $url;							# Self referencing link

my @list;							# File selection
my @pass;							# Passed params
my @upload;							# Uploaded files

my %args;							# Embeded parameters
my %home;							# Home page hash


#################################################################################
# Program Area
#################################################################################

	# Initial handles
	$cgi      = new CGI;	
	$database = new eRealms::Database();
	$table    = new eRealms::Table();

	# Connect and fetch
	$database->Connect();

	# Retreive hash
	$database->GetHashHome(\%home);


	# Advanced retreivals
	# Handle uploads
	for (my $cnt=1; $cnt <= $home{'SetUpload'}; $cnt++)
	{
		# Add to array
		$upload[$cnt] = $cgi->upload("FUPLOAD$cnt");
	}

	# Removals
	@list = $cgi->param('FSELECT');	


	# Homepage and Parameter instances
	$hp    = new eRealms::Home($database, $home{'SetRoot'});
	$param = new eRealms::Param($database, $cgi);
	$param->GetParam();

	# Create instance
	$tmpl = new eRealms::Template(\%home);


	# Document header
	print $cgi->header();
	$database->DocumentGetGalleryHeader();

	# Should system be active
	if ($home{'SetActive'})
	{
		# Login handle
		$login = new eRealms::Login($database, $cgi, $param, \%home);

		# Verify login and continue with system
		if ($login->GetVerificationNormal())
		{
			# Expired or not?
			($expire) = $database->DataGetPuppeteerExpire($param->{'USER'});

			# Account still valid
			if ($expire eq 'no')
			{
				# Quick param
				@pass = ($database, $cgi, $param, $table, $tmpl, $hp, \%home);


				# Make sure Directory is sane 
				if (DirHandler(@pass))
				{
					# Fetch self referencing link
					$url = $cgi->url(-absolute=>1);

					# Base parameters
					%args = (
						USER  => $param->{'USER'},
						CRYPT => $param->{'CRYPT'}
					 );

					# Appropriate parameters
					my $imain = $param->EmbedInline(%args);
					my $ilist = $param->EmbedInline(%args, LISTING=>'True');
					my $isize = $param->EmbedInline(%args, RESIZER=>'True');
					my $iupld = $param->EmbedInline(%args, UPLOADS=>'True');


					print "<CENTER><A HREF=\"$url$imain\">$home{TagTopMain}</A> ",
					      "<A HREF=\"$url$ilist\">$home{TagTopList}</A> ",
					      "<A HREF=\"$url$isize\">$home{TagTopResize}</A> ",
					      "<A HREF=\"$url$iupld\">$home{TagTopUpload}</A></CENTER>\n<BR><BR>\n";


					# Directory listing
					if (defined($param->{'LISTING'}))
					{
						# Display title
						$tmpl->ShowMain($home{'TagListing'});

						# Display information
						DirListing(@pass);
					}

					# Image resizing
					elsif (defined($param->{'RESIZER'}))
					{
						# Display title
						$tmpl->ShowMain($home{'TagResize'});

						# Handling of folder
						FileResize(@pass, \@list);
					}


					# File uploads
					elsif (defined($param->{'UPLOADS'}))
					{
						# Display title
						$tmpl->ShowMain($home{'TagAdd'});

						# Handling of uploads
						FileUpload(@pass);
					}

					else
					{
						# Display title
						$tmpl->ShowMain($home{'TagTitle'});

						# Handling of folder
						if (FileEdit(@pass, \@list))
						{
							# Default display
							HomeHandler(@pass, \@upload, \@list); 
						}
					}
				}
			}
			else
			{
				# Display disabled message
				$table->MakeTop();
				$table->MakeSingle($home{'MsgExpire'});
				$table->MakeBottom();
			}
		}
	}
	else
	{
		# Display disabled message
		$table->MakeTop();
		$table->MakeSingle($home{'MsgDisabled'});
		$table->MakeBottom();
	}

	# Document footer
	$database->DocumentGetGalleryFooter();


#################################################################################
# Sub-Routines
#################################################################################

#####################
# Directory Handler 
#
# This will make sure that certain components of eRealmss Home page system,
# primarily the home page directory is in order.   This means creation, prompt
# or whatever is needed to get it going. 

sub DirHandler 
{
	#####################
	# Data members
	my $database = shift; 					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	my $table    = shift;					# Table handler 
	my $tmpl     = shift;					# Template handle
	my $hp       = shift;					# Homepage handle
	my $home     = shift;					# Home page hash handle 
	my $message  = shift;					# Error message

	my $data     = $database->{'HANDLE'};			# Database connection handle
	my $dir;						# Directory

	my $header;						# Header file
	my $readme;						# Readme file


	#####################
	# Program area

	# Retreive DIR
	($dir) = $database->DataGetPuppeteerHome($param->{'USER'});

 	# Determin status 
	# Directory defined 
	if (defined($dir))
	{
		# Ensure existence
		unless (-d "$home->{SetRoot}$dir")
		{
			# Create and verify
			unless (mkdir("$home->{SetRoot}$dir", 0775))
			{
				# Display error message
				$table->MakeTop();
				$table->MakeSingle($home->{'MsgFail'});
				$table->MakeBlank();
				$table->MakeSingle("<CENTER>$!</CENTER>");
				$table->MakeBottom();

				# Reset Database
				$data->do("UPDATE Puppeteer 
					SET   PuppeteerHome=NULL 
					WHERE PuppeteerLogin=?", {}, 

				 $param->{'USER'});	
				
				# Return False
				return 0;
			} 
		}

		# Set Basedir
		$hp->{'BASEDIR'} = "$home->{SetRoot}$dir";


		# Retreive header and readme
		($header) = $database->DocumentGetHomeHeader();
		($readme) = $database->DocumentGetHomeReadme();

		# Write contents
		$hp->FileWrite($home->{'SetHEADER'}, $header);
		$hp->FileWrite($home->{'SetREADME'}, $readme);

		# Return true
		return 1;
	}

	# Directory creation
	elsif ((defined($param->{'DCREATE'})) && ($param->{'DCREATE'} !~ /\W+/g))
	{
		# Parameter handling
		$dir = $param->{'DCREATE'};
		delete $param->{'DCREATE'};


		# Ensure existence
		unless (-d "$home->{SetRoot}$dir")
		{
			# Create and verify
			# Successful
			if (mkdir("$home->{SetRoot}$dir", 0755))
			{
				# Retreive header and readme
				($header) = $database->DocumentGetHomeHeader();
				($readme) = $database->DocumentGetHomeReadme();

				# Write contents
				$hp->FileWrite("$home->{SetRoot}$dir/$home->{SetHEADER}", $header);
				$hp->FileWrite("$home->{SetRoot}$dir/$home->{SetREADME}", $readme);

				# Update Database
				$data->do("UPDATE Puppeteer 
					SET   PuppeteerHome=? 
					WHERE PuppeteerLogin=?", {}, 
				 $dir,
				 $param->{'USER'});
				
				# Return True
				return 1;
			} 

			# Fail
			else
			{
				# Display error message
				$table->MakeTop();
				$table->MakeSingle($home->{'MsgFail'});
				$table->MakeBlank();
				$table->MakeSingle("<CENTER>$!</CENTER>");
				$table->MakeBottom();
			}
		}
		else
		{ DirHandler($database, $cgi, $param, $table, $tmpl, $hp, $home, $home->{'MsgExists'}); }
		
		# Return false
		return 0; 
	} 

	# New page setup
 	else	
	{
		# Start form
		print $cgi->start_form();
		print $param->EmbedNormal($param->Flat()), "\n";

		# Display creation form
		$table->MakeTop();

		# Message display
		if (defined($message)) 
		{
			$table->MakeSingle($message); 
			$table->MakeBlank($cgi);
		}

		$table->MakeSingle($tmpl->PassTitle($home->{'TagHome'}));
		$table->MakeSingle($cgi->textfield('DCREATE', '', 55, 55));
		$table->MakeBlank();
		$table->MakeValid('&nbsp;', $cgi->submit($home->{'TxtCreate'}));
		$table->MakeBottom();

		# Embed and close form
		print $cgi->end_form(), "\n";

		# Return false
		return 0;
	}
}

#####################
# Directory Listing
#
# Used to show a user all files in the folder and allow them to select it
# if needed.  This is used if directory indexing has been disabled.

sub DirListing
{
	#####################
	# Data members
	my $database = shift; 					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	my $table    = shift;					# Table handle
	my $tmpl     = shift;					# Template handle
	my $hp       = shift;					# Homepage handle
	my $home     = shift;					# Home page hash handle 

	my $bytes;						# Individual size
	my $count;						# File count
	my $dir;						# Users Directory
	my $file;						# Individual file
	my $size;						# Size of files

	my @files;						# File listing
	my @listing;						# Listing of files


	#####################
	# Program Area

	# Cleanup
	$param->Cleanup('LISTING');


	# Data retreival
	# Retreive DIR
	($dir) = $database->DataGetPuppeteerHome($param->{'USER'});

	# Base directory
	$hp->{'BASEDIR'} = $home->{'SetRoot'};


	# Retreive file listing
	($size, $count) = $hp->FileListing($dir, \@files, $home); 


	# Top of table
	$table->MakeTop();

	# Url listing
	$table->MakeSingle($tmpl->PassSub($home->{'TagUrl'}));
	$table->MakeSingle("<A HREF=\"$home->{SetUrl}$dir\" TARGET=\"_new\">$home->{SetUrl}$dir</A>");
	$table->MakeBlank();

	# Directory listing
	$table->MakeSingle($tmpl->PassSub($home->{'TagListing'}));

	# Cycle and list files
	foreach $file (sort(@files))
	{
		# Retreive file size
		$bytes = -s "$home->{SetRoot}$dir/$file";

		# Filename is escaped
		$file = $cgi->escape($file);

		# Display Filename
		$table->MakeSingle("<A HREF=\"$home->{SetUrl}$dir/$file\" TARGET=\"_new\">$home->{SetUrl}$dir/$file</A> ($bytes Bytes)");
	}

	# Useage Summary
	$table->MakeBlank();
	$table->MakeSingle($tmpl->PassSub($home->{'TagSummary'}));
	$table->MakeSingle("$size $home->{TagStatusSize} $count $home->{TagStatusCount}");
	$table->MakeBottom();
}


#####################
# File Conflict
#
# File conflict is there to prevent any chance of files being overriden
# by file uploads and resize handling.

sub FileConflict
{
	#####################
	# Data members
	my $dir  = shift;					# Directory name
	my $file = shift; 					# Filename

	my $fn;							# Filename
	my $num = 0;						# Number


	#####################
	# Program area

	# Acceptable file name
	if (-f "$dir/$file")
	{
		do
		{
			# Increment
			$num++;

			# Assign
			$fn = $file; 

			# Rename
			$fn =~ s/^(.+)\.(.*)$/$1-$num.$2/s;
		} 
	 	while (-f "$dir/$fn");

		# Re-assign
		$file = $fn;
	}

	# Return
	return $file;
}


#####################
# File Edit 
#
# File edit, will be called before the default script and will be used in cases
# where the option to edit a file has been clicked.   If it is not needed
# it will return true and the default will kick in. 

sub FileEdit 
{
	#####################
	# Data members
	my $database = shift; 					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	my $table    = shift;					# Table handler
	my $tmpl     = shift;					# Template handler
	my $hp       = shift;					# Homepage handle
	my $home     = shift;					# Home page hash handle 
	my $list     = shift;					# Selected files

	my $contents;						# Contents of file
	my $dir;						# Directory name
	my $file;						# File name
	my $message;						# Status message


	#####################
	# Program area

	# Initial check
	return 1 unless (defined($param->{$home->{'TxtEdit'}}));


	# Retreive directory handle 
	($dir)  = $database->DataGetPuppeteerHome($param->{'USER'});

	# Filename
	($file) = $param->{'FSELECT'};


	# Set base directory
	$hp->{'BASEDIR'} = "$home->{SetRoot}/$dir";


	# Operations
	# Resize
	if (defined($param->{'FEDIT'}))
	{
		# Correct size
	 	if (length($param->{'FEDIT'}) > $home->{'SetEditMin'})
		{
			# Write file	
			$hp->FileWrite($file, $param->{'FEDIT'});

			# Message display
			$message = $home->{'MsgEdit'};	
		}

		else
		{
			# Warn of min file size
			$message = $tmpl->Pass('MsgChar', 'MCHAR', $home->{'SetEditMin'});
		}

		# Cleanup
		$param->Cleanup('FEDIT');
	}


	# File load and display
	if ((defined($file))
	 && ($file =~ /\.(htm|html|shtml|txt)$/i)
	 && (open(EDIT, "$hp->{BASEDIR}/$file")))
	{
		# Read from file
		while (<EDIT>) { $contents = "$contents$_"; }

		# Close
		close(EDIT);


		# HTML Display
		# Start form
		print $cgi->start_form();
		print $param->EmbedNormal(
			USER=>$param->{'USER'},
			CRYPT=>$param->{'CRYPT'},
			FSELECT=>$file
		 );

		# Table display
		$table->MakeTop();

		# Message
		if (defined($message)) 
		{ 
			$table->MakeSingle($message); 
			$table->MakeBlank(); 
		}

		# File listing
		$table->MakeSingle($tmpl->PassSub($home->{'TagEdit'}));
		$table->MakeSingle($cgi->textarea(
			-name=>'FEDIT',
			-rows=>10,
			-columns=>75,
			-wrap=>'OFF',
			-default=>$contents)
		 );

		# Table bottom
		$table->MakeBlank();
		$table->MakeSingle($cgi->submit($home->{'TxtEdit'}));
		$table->MakeBottom();

		# Embed and close form
		print $cgi->end_form(), "\n";


		# Retrun false
		return 0;
	}

	# Retrun true
	return 1;
}


#####################
# File Resize 
#
# File resize will allow a user to select a file and resize it to an appropriate
# demension.   Due to the nature of this, some steps will be taken to only allow
# images, though nothing is certain.

sub FileResize 
{
	#####################
	# Data members
	my $database = shift; 					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	my $table    = shift;					# Table handler
	my $tmpl     = shift;					# Template handler
	my $hp       = shift;					# Homepage handle
	my $home     = shift;					# Home page hash handle 
	my $list     = shift;					# Selected files

	my $side;						# Left hand side
	my $sorted;						# Sorted file CGI

	my $dir;						# Directory name
	my $file;						# File name
	my $image;						# Image handle
	my $message;						# Status message

	my @files;						# Files list
	my @sorted;						# Sorted files


	#####################
	# Program area

	# Retreive directory handle 
	($dir) = $database->DataGetPuppeteerHome($param->{'USER'});

	# Set basedir
	$hp->{'BASEDIR'} = "$home->{SetRoot}/$dir";


	# Operations
	# Resize
	if (defined($param->{$home->{'TxtResize'}}))
	{
		# First file name
		my $file = shift(@{$list});

		# Safety checks
		if (($file ne $home->{'TxtNone'})
		 && ($param->{'FMAX'} =~ /^\d+$/)
		 && ($param->{'FMAX'} > 0)
		 && ($param->{'FMAX'} <= $home->{'SetResize'}))
		{
			# Create instance
			$image = new eRealms::Image($database, $hp->{'BASEDIR'});


			# Create copy
			# Filenames
			my $old = $file;
			my $new = FileConflict($hp->{'BASEDIR'}, $file);

			# Copy to resize
			copy("$hp->{BASEDIR}/$old", "$hp->{BASEDIR}/$new");


			# Resize
			$image->Set($new, $param->{'FMAX'});

			# Set Message
			$message = $home->{'MsgResize'};
		}


		# Cleanup
		$param->Cleanup('FMAX','FSELECT');
	}



	# Reset basedir
	$hp->{'BASEDIR'} = $home->{'SetRoot'};


	# Data handling
	# Retreive file listing
	$hp->FileListing($dir, \@files, $home); 

	# Exclude
	foreach $file (@files)
	{
		# Push onto array
		# Specific types only
		push(@sorted, $file) if $file =~ /\.(gif|jpe|jpeg|jpg|pcx|psd|png|tiff|tga)$/i;
	}


	# Sort and prepare 
	@sorted = sort(@sorted);
	unshift(@sorted, $home->{'TxtNone'});

	$sorted = $cgi->scrolling_list('FSELECT', \@sorted, $sorted[0], 15);


	# Generate widget
	$side = $tmpl->PassSub($home->{'TagMaxSize'}) . '<BR>' 
	  . $cgi->textfield('FMAX', '300', 5, 3) . '&nbsp;&nbsp;' 
	  . $cgi->submit($home->{'TxtResize'});


	# HTML Display
	# Start form
	print $cgi->start_form();
	print $param->EmbedNormal($param->Flat()), "\n";

	# Table display
	$table->MakeTop();

	# Message
	if (defined($message)) 
	{ 
		$table->MakeSingle($message); 
		$table->MakeBlank(); 
	}

	# File listing
	$table->MakeSingle($tmpl->PassSub($home->{'TagListing'}));
	$table->MakeValid($sorted, $side, 'middle');

	# Table bottom
	$table->MakeBottom();

	# Embed and close form
	print $cgi->end_form(), "\n";
}


#####################
# File Upload
#
# File upload window, which will allow the main window to handle the
# entire process.  As a result, this window is literally a one time
# only operation.

sub FileUpload
{
	#####################
	# Data members
	my $database = shift; 					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	my $table    = shift;					# Table handle
	my $tmpl     = shift;					# Template handle
	my $hp       = shift;					# Homepage handle
	my $home     = shift;					# Home page hash handle 

	my $cnt;						# File count
	my $dir;						# Users Directory
	my $size;						# Space used

	my @files;						# File listing


	#####################
	# Program Area

	# Cleanup
	$param->Cleanup('UPLOADS');


	# Data retreival
	# Retreive DIR
	($dir) = $database->DataGetPuppeteerHome($param->{'USER'});

	# Base directory
	$hp->{'BASEDIR'} = $home->{'SetRoot'};


	# Retreive file listing
	($size) = $hp->FileListing($dir, \@files, $home); 

	# Start form
	print $cgi->start_multipart_form();
	print $param->EmbedNormal($param->Flat()), "\n";

	# Top of table
	$table->MakeTop();

	# Addition of files
	if ($size < ($home->{'SetSize'} * 1048576))
	{
		# Cycle through
		for ($cnt=1; $cnt <= $home->{'SetUpload'}; $cnt++)
		{
			# Display addition area
			$table->MakeValid(
				$tmpl->PassSub($home->{'TagUpload'}, MNUM=>$cnt),
				$cgi->filefield("FUPLOAD$cnt", '', 40, 140)
			 );
		}

		# Button
		$table->MakeBlank();
		$table->MakeSingle($cgi->submit($home->{'TxtCreate'}));
	}
	# Display lack of capability
	else { $table->MakeSingle($home->{'MsgSpace'}); }


	# Bottom
	$table->MakeBottom();

	# End form
	print $cgi->end_form(), "\n";
}


#####################
# HomeHandler 
#
# Provides the interface to see all files within their home folders as well
# as allow for additions and removals as one sees fit.  

sub HomeHandler
{
	#####################
	# Data members
	my $database = shift; 					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	my $table    = shift;					# Table handler
	my $tmpl     = shift;					# Template handler
	my $hp       = shift;					# Homepage handle
	my $home     = shift;					# Home page hash handle 
	my $upload   = shift;					# File upload hash
	my $list     = shift;					# Selected files

	my $button;						# Buttons used
	my $count;						# File listing
	my $dir;						# Directory
	my $size;						# File size
	my $sorted;						# Sorted file CGI

	my $contents;						# File contents
	my $file;						# File name
	my $handle;						# File handle
	my $type;						# File type

	my $err;						# Error message
	my $message;						# Status message

	my @files;						# Files list
	my @sorted;						# Sorted files


	#####################
	# Program area

	# Retreive directory handle 
	($dir) = $database->DataGetPuppeteerHome($param->{'USER'});

	# Set basedir
	$hp->{'BASEDIR'} = "$home->{SetRoot}/$dir";


	# Operations
	# Upload field handle
	for (my $cnt=1; $cnt <= $home->{'SetUpload'}; $cnt++)
	{
		# Create handle
		$handle = $upload->[$cnt];

		# Worth a look?
		if (defined($handle))
		{	
			# Retreive MIME Type
			$type = $cgi->uploadInfo($param->{"FUPLOAD$cnt"})->{'Content-Type'};

			# Filename handling
			$file = $param->{"FUPLOAD$cnt"}; 
			$file =~ m!([^/:\\]*)$!;
			$file = $1;  

			# Only allow specific MIME types
			if (($type =~ /^image/i) || ($type =~ /^text/i))
			{
				unless (($file =~ /^\./)
				  || ($file eq $home->{'SetHEADER'})
				  || ($file eq $home->{'SetREADME'}))
				{
					# Acceptable file name
					$file = FileConflict("$home->{SetRoot}/$dir", $file);


					# Retreive information
					$contents = '';
					while (<$handle>) { $contents = "$contents$_"; }
			
					# Write information
					$err = $hp->FileWrite($file, $contents); 

					# Happy land
					$message = $home->{'MsgCreate'};
				}
				# Otherwise error message
				else
				{ $message = $home->{'MsgFail'}; }
			}
			# Otherwise error message
			else
			{ $message = $home->{'MsgFail'}; }
		}

		# Cleanup
		$param->Cleanup("FUPLOAD$cnt", $home->{'TxtCreate'});
	}

	# Remove files
	if ((defined($param->{$home->{'TxtDelete'}}))
	 && (defined($param->{'CONFIRM'})))
	{
		# Cycle
		foreach $file (@{$list})
		{
	 		if ($file ne $home->{'TxtNone'})
			{
				# Remove file
				$hp->FileUnlink($file);

				# Assign message
				$message = $home->{'MsgDelete'};
			}
		}

		# Cleanup
		$param->Cleanup('FSELECT', $home->{'TxtDelete'});	
	}

	# Always clean
	$param->Cleanup('CONFIRM');


	# Rename
	if (defined($param->{$home->{'TxtRename'}}))
	{
		# First file name
		my $new = $param->{'FRENAME'};
		my $old = shift(@{$list});

		# Safety checks
		if (($old ne $home->{'TxtNone'})
		 && ($new =~ /^\w[\w\.\-]+$/)
		 && (!-f "$home{SetRoot}/$dir/$new"))
		{
			# Rename file
			rename("$home{SetRoot}/$dir/$old", "$home{SetRoot}/$dir/$new");

			# Assign message
			$message = $home->{'MsgRename'};
		}

		# Cleanup
		$param->Cleanup('FSELECT', $home->{'TxtRename'});
	}



	# Reset basedir
	$hp->{'BASEDIR'} = $home->{'SetRoot'};


	# Data handling
	# Retreive file listing
	($size, $count) = $hp->FileListing($dir, \@files, $home); 

	# Sort and prepare 
	@sorted = sort(@files);
	unshift(@sorted, $home->{'TxtNone'});

	$sorted = $cgi->scrolling_list('FSELECT', \@sorted, $sorted[0], 15, 'true');


	# Generate widget
	$button = $cgi->submit($home->{'TxtDelete'}) . ' &nbsp; ' 
	  . $cgi->checkbox(-name=>'CONFIRM', -label=>'') . ' <BR><BR> '
	  . $cgi->submit($home->{'TxtEdit'});


	# HTML Display
	# Start form
	print $cgi->start_form();
	print $param->EmbedNormal($param->Flat()), "\n";

	# Table display
	$table->MakeTop();

	# Message
	if (defined($message)) 
	{ 
		$table->MakeSingle($message); 
		$table->MakeBlank(); 
	}

	# File listing
	$table->MakeSingle($tmpl->PassSub($home->{'TagListing'}));
	$table->MakeValid($sorted, $button, 'middle');

	# Rename
	$table->MakeBlank();
	$table->MakeSingle($tmpl->PassSub($home->{'TagRename'}));
	$table->MakeValid($cgi->textfield('FRENAME', '', 35, 35), $cgi->submit($home->{'TxtRename'}));

	# Useage Summary
	$table->MakeBlank();
	$table->MakeSingle($tmpl->PassSub($home->{'TagSummary'}));
	$table->MakeSingle("$size $home->{TagStatusSize} $count $home->{TagStatusCount}");
	$table->MakeBottom();

	# Embed and close form
	print $cgi->end_form(), "\n";
}
