#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 18-Dec-2003
#################################################################################
#
# Realm - Script part of Ethereal Realms designed to interact within realms
# Copyright (C) 2000-2004  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 29-26503 TWP RD 511
#                 Spruce Grove, Alberta
#                 T7Y 1G4
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement

use eRealms::Comm;						# Communication handler
use eRealms::Database;						# Database handler
use eRealms::Login;						# Login functionality
use eRealms::Param;						# Parameter control
use eRealms::Post;						# Post handler
use eRealms::Pull;						# Post extracts
use eRealms::Who;						# Who is on



#################################################################################
# Data Members
#################################################################################
my $cgi;							# Common gateway interface handle
my $comm;							# Communications panel handle
my $database;							# Database handle
my $login;							# Login handle
my $param;							# Parameter handle
my $pull;							# Pull handle
my $post;							# Post handle

my $chatpref;							# Chat preferences
my $icomm;							# Comm panel inline parameters
my $imain;							# Main panel inline parameters
my $javascript;							# Javascript preferences
my $stm;							# Stream URL
my $url;							# Basic URL

my $cols;							# Columns
my $rows;							# Rows

my %realm;							# Realm hash

#################################################################################
# Program Area
#################################################################################

	# Initial handle
	$cgi      = new CGI;	
	$database = new eRealms::Database();

	# Connect and fetch
	$database->Connect();


	# Parameter handling
	$param    = new eRealms::Param($database, $cgi);
	$param->GetParam();

	# Fetch hash
	$database->GetHashRealm(\%realm);

	# Fetch URL
	$url = $cgi->url(-absolute=>1);

	# Stream URL
	$stm = $url;
	$stm =~ s/^(.+)\/.+$/$1\/stream.pl/gs;

	
	# Login handle
	$login = new eRealms::Login($database, $cgi, $param);

	# Authenthication
	if ($login->GetVerificationRealm())
	{
		# Make sure logins are possible
		$login->CheckLegality(\%realm);

		# Realm specific handles
		$comm  = new eRealms::Comm($database, $cgi);
		$post  = new eRealms::Post();
		$pull  = new eRealms::Pull();


		# Verify for need for extract
		$pull->Posts($database, $cgi, $param);
	

		# Retreive javascript and chat preferences
		($javascript, $chatpref) = $database->DataGetChatOptions($param->{'USER'});

		# Handle post writing
		$comm->DataWrite($param);


		# Framed chat
		if ($chatpref eq 'framed')
		{
			# Only do following if called with no parameters
			unless ((defined($param->{'MAIN'})) 
			     || (defined($param->{'COMM'})))
			{
				# Retreive parameter list
				$icomm = $param->EmbedInline($param->Flat());
				$imain = $param->EmbedInline(
					USER  => $param->{'USER'},
					CRYPT => $param->{'CRYPT'},
					ROOM  => $param->{'ROOM'}
				 );

				# Retreive frameset
				($cols, $rows) = $database->DataGetRealmFrameset($param->{'ROOM'}); 



				# Print file header
				print $cgi->header();

				# Print header
				print "<HTML>\n<HEAD>\n <TITLE>$param->{'ROOM'}</TITLE>\n</HEAD>\n\n";

				# Print frameset
				print "<FRAMESET ID=\"resize\"  ROWS=\"*,$rows\">\n";
				print " <FRAME NAME=\"Main\" SRC=\"$url$imain&MAIN=True\">\n";
				print " <FRAME NAME=\"Comm\" SRC=\"$url$icomm&COMM=True\">\n";
				print "</FRAMESET>\n";

				# End of HTML
				print "</HTML>\n";
			}
	
			# Message display area
			if (defined($param->{'MAIN'}))
			{ 
				# Cleanup
				$param->Cleanup('CHAR');


				# File and HTML header
				print $cgi->header(-target=>'Main');
				print $database->FetchRealmHeader($param->{'ROOM'});

				# Display banner
				ShowBanner($database, $param);


				# Posting related
				$post->HandlePosts($database, $cgi,  $param, 'TRUE'); 

				# Parameter handler
				$param->Cleanup('MAIN');


				# Activation button
				print $cgi->start_form();
				print $param->EmbedNormal($param->Flat(), MAIN=>'True'), "\n";
				

				# Display refresh
				$post->RefreshFramed($database, $cgi, $param);


				print $cgi->end_form(), "\n";
				print $database->FetchRealmFooter($param->{'ROOM'});
			}

			# Comm panel
			if (defined($param->{'COMM'}))
			{ 
				# Communications panel
				$comm->CommIndependant($param); 
			}
		}


		# Streaming chat with autoscroll
		elsif (($chatpref eq 'autoscroll') && ($javascript eq 'yes')) 
		{
			# Only do following if called with no parameters
			unless ((defined($param->{'MAIN'}))
			 || (defined($param->{'COMM'}))
			 || (defined($param->{'SCRL'})))
			{
				# Retreive parameter list
				$icomm = $param->EmbedInline($param->Flat());
				$imain = $param->EmbedInline(
					USER  => $param->{'USER'},
					CRYPT => $param->{'CRYPT'},
					ROOM  => $param->{'ROOM'}
				 );

				# Retreive frameset
				($cols, $rows) = $database->DataGetRealmFrameset($param->{'ROOM'}); 


				# Print file header
				print $cgi->header();				
				
				# Print header
				print "<HTML>\n<HEAD>\n <TITLE>$param->{'ROOM'}</TITLE>\n</HEAD>\n\n";

				# Print frameset
				print "<FRAMESET COLS=\"$cols,*\">\n";
				print " <FRAME NAME=\"Remote\" SRC=\"$url$imain&SCRL=True\">\n";
				print " <FRAMESET ID=\"resize\" ROWS=\"*,$rows\">\n";
				print "  <FRAME NAME=\"Main\" SRC=\"$stm$imain&MAIN=True\">\n";
				print "  <FRAME NAME=\"Comm\" SRC=\"$url$icomm&COMM=True\">\n";
				print " </FRAMESET>\n";
				print "</FRAMESET>\n";

				# End of HTML
				print "</HTML>\n";
			}	
	
			# Message display area
			if (defined($param->{'MAIN'}))
			{ 
				# Cleanup
				$param->Cleanup('CHAR');


				# Flush standard output				
				$| = 1;
				print "";

				# File header
				print $cgi->header(-target=>'Main');
				print $database->FetchRealmHeader($param->{'ROOM'});

				# Display banner
				ShowBanner($database, $param);


				# Java enabled post window
				$post->HandleStream($database, $cgi, $param); 

				# Document footer
				print $database->FetchRealmFooter($param->{'ROOM'});
			}

			# Comm panel
			if (defined($param->{'COMM'}))
			{
				$comm->CommAutoScroll($param);
			}

			# Remote control
			if (defined($param->{'SCRL'}))
			{
				print $cgi->header(-target=>'Remote');
				print $database->FetchRealmRemote($param->{'ROOM'});
			}
		}

		# Streaming chat
		elsif (($chatpref eq 'stream') || ($chatpref eq 'autoscroll'))
		{
			# Only do following if called with no parameters
			unless ((defined($param->{'MAIN'}))
			     || (defined($param->{'COMM'})))
			{
				# Retreive parameter list
				$icomm = $param->EmbedInline($param->Flat());
				$imain = $param->EmbedInline(
					USER  => $param->{'USER'},
					CRYPT => $param->{'CRYPT'},
					ROOM  => $param->{'ROOM'}
				 );

				# Retreive frameset
				($cols, $rows) = $database->DataGetRealmFrameset($param->{'ROOM'}); 


				# Print file header
				print $cgi->header();				
				
				# Print header
				print "<HTML>\n<HEAD>\n <TITLE>$param->{'ROOM'}</TITLE>\n</HEAD>\n\n";

				# Print frameset
				print "<FRAMESET ID=\"resize\" ROWS=\"*,$rows\">\n";
				print "  <FRAME NAME=\"Main\" SRC=\"$stm$imain&MAIN=True\">\n";
				print "  <FRAME NAME=\"Comm\" SRC=\"$url$icomm&COMM=True\">\n";
				print "</FRAMESET>";

				# End of HTML
				print "</HTML>\n";
			}	
	
			# Message display area
			if (defined($param->{'MAIN'}))
			{ 
				# Cleanup
				$param->Cleanup('CHAR');


				# Flush standard output				
				$| = 1;
				print "";

				# File header
				print $cgi->header(-target=>'Main');
				print $database->FetchRealmHeader($param->{'ROOM'});

				# Display banner
				ShowBanner($database, $param);


				# Posting related
				$post->HandleStream($database, $cgi, $param); 

				# Document footer
				print $database->FetchRealmFooter($param->{'ROOM'});
			}

			# Comm panel
			if (defined($param->{'COMM'}))
			{ 
				$comm->CommStream($param); 
			}
		}


		# Basic chat system
		else
		{
			# Print file and HTML header
			print $cgi->header();
			print $database->FetchRealmHeader($param->{'ROOM'});

			# Display banner
			ShowBanner($database, $param);


			# Message display and posting
			$post->HandlePosts($database, $cgi, $param); 

			# Spacer
			print "<BR><BR>\n";

			$comm->CommDependant($param);

			# Document footer
			print $database->FetchRealmFooter($param->{'ROOM'});
		}
	}


#################################################################################
# Sub-Routines
#################################################################################

#####################
# Show Banner
#
# Very simple wrapper around the banner display which will only invoke the
# call to the database if the user has not yet been in the realm specifically.
# This reduces the load on the database and bandwidth consumption as a result.

sub ShowBanner
{
	####################
	# Data members

	my $database = shift;					# Database handle
	my $param    = shift;					# Parameter list


	####################
	# Program Area

	# First time
	unless (defined($param->{'ROOT'})) 
	{
		# Display banner
		print $database->FetchRealmBanner($param->{'ROOM'});
	}
}
